#pragma once

// Hardware gospel from:
//
// https://www.analog.com/media/en/technical-documentation/data-sheets/MAX98091.pdf

#define MAX98091_BITS(n, v) \
    (((v) & ((1 << MAX98091_##n##_WIDTH) - 1)) << (MAX98091_##n##_BASE))
#define MAX98091_BOOL(n, v) MAX98091_BITS(n, v ? 1 : 0)
#define MAX98091_ON(n) MAX98091_BOOL(n, true)
#define MAX98091_OFF(n) MAX98091_BOOL(n, false)

#define MAX98091_SOFTWARE_RESET 0x00

#define MAX98091_JACK_STATUS 0x02
#define MAX98091_JACK_STATUS_JKSNS_BASE 1
#define MAX98091_JACK_STATUS_JKSNS_WIDTH 1
#define MAX98091_JACK_STATUS_LSNS_BASE 2
#define MAX98091_JACK_STATUS_LSNS_WIDTH 1

#define MAX98091_DAI_INTERFACE 0x06

#define MAX98091_DAC_PATH 0x07

#define MAX98091_LINE_TO_ADC 0x09

#define MAX98091_LINE_INPUT_CONFIG 0x0d

#define MAX98091_INPUT_MODE 0x0f

#define MAX98091_MIC1_INPUT_LEVEL 0x10
#define MAX98091_MIC1_INPUT_LEVEL_PGAM1_BASE 0
#define MAX98091_MIC1_INPUT_LEVEL_PGAM1_WIDTH 5
#define MAX98091_MIC1_INPUT_LEVEL_PA1EN_BASE 5
#define MAX98091_MIC1_INPUT_LEVEL_PA1EN_WIDTH 2

#define MAX98091_DIGITAL_MIC_ENABLE 0x13
#define MAX98091_DIGITAL_MIC_ENABLE_DIGMIC1L_BASE 0
#define MAX98091_DIGITAL_MIC_ENABLE_DIGMIC1L_WIDTH 1
#define MAX98091_DIGITAL_MIC_ENABLE_DIGMIC1R_BASE 1
#define MAX98091_DIGITAL_MIC_ENABLE_DIGMIC1R_WIDTH 1
#define MAX98091_DIGITAL_MIC_ENABLE_DIGMIC2L_BASE 2
#define MAX98091_DIGITAL_MIC_ENABLE_DIGMIC2L_WIDTH 1
#define MAX98091_DIGITAL_MIC_ENABLE_DIGMIC2R_BASE 3
#define MAX98091_DIGITAL_MIC_ENABLE_DIGMIC2R_WIDTH 1
#define MAX98091_DIGITAL_MIC_ENABLE_DMICCLK_BASE 4
#define MAX98091_DIGITAL_MIC_ENABLE_DMICCLK_WIDTH 3

#define MAX98091_LEFT_ADC_MIXER 0x15
#define MAX98091_LEFT_ADC_MIXER_LINE_IN_A_BASE 3
#define MAX98091_LEFT_ADC_MIXER_LINE_IN_A_WIDTH 1
#define MAX98091_LEFT_ADC_MIXER_LINE_IN_B_BASE 4
#define MAX98091_LEFT_ADC_MIXER_LINE_IN_B_WIDTH 1
#define MAX98091_LEFT_ADC_MIXER_MICROPHONE_1_BASE 5
#define MAX98091_LEFT_ADC_MIXER_MICROPHONE_1_WIDTH 1

#define MAX98091_RIGHT_ADC_MIXER 0x16
#define MAX98091_RIGHT_ADC_MIXER_LINE_IN_A_BASE 3
#define MAX98091_RIGHT_ADC_MIXER_LINE_IN_A_WIDTH 1
#define MAX98091_RIGHT_ADC_MIXER_LINE_IN_B_BASE 4
#define MAX98091_RIGHT_ADC_MIXER_LINE_IN_B_WIDTH 1
#define MAX98091_RIGHT_ADC_MIXER_MICROPHONE_1_BASE 5
#define MAX98091_RIGHT_ADC_MIXER_MICROPHONE_1_WIDTH 1

#define MAX98091_SYSTEM_CLOCK 0x1B
#define MAX98091_SYSTEM_CLOCK_PSCLK_BASE 4
#define MAX98091_SYSTEM_CLOCK_PSCLK_WIDTH 2

#define MAX98091_IO_CONFIGURATION 0x25

#define MAX98091_FILTER_CONFIGURATION 0x26

#define MAX98091_LEFT_HP_MIXER 0x29
#define MAX98091_LEFT_HP_MIXER_LEFT_DAC_BASE 0
#define MAX98091_LEFT_HP_MIXER_LEFT_DAC_WIDTH 1
#define MAX98091_LEFT_HP_MIXER_RIGHT_DAC_BASE 1
#define MAX98091_LEFT_HP_MIXER_RIGHT_DAC_WIDTH 1
#define MAX98091_LEFT_HP_MIXER_LINE_A_BASE 2
#define MAX98091_LEFT_HP_MIXER_LINE_A_WIDTH 1
#define MAX98091_LEFT_HP_MIXER_LINE_B_BASE 3
#define MAX98091_LEFT_HP_MIXER_LINE_B_WIDTH 1

#define MAX98091_RIGHT_HP_MIXER 0x2A
#define MAX98091_RIGHT_HP_MIXER_LEFT_DAC_BASE 0
#define MAX98091_RIGHT_HP_MIXER_LEFT_DAC_WIDTH 1
#define MAX98091_RIGHT_HP_MIXER_RIGHT_DAC_BASE 1
#define MAX98091_RIGHT_HP_MIXER_RIGHT_DAC_WIDTH 1
#define MAX98091_RIGHT_HP_MIXER_LINE_A_BASE 2
#define MAX98091_RIGHT_HP_MIXER_LINE_A_WIDTH 1
#define MAX98091_RIGHT_HP_MIXER_LINE_B_BASE 3
#define MAX98091_RIGHT_HP_MIXER_LINE_B_WIDTH 1

#define MAX98091_HPCONTROL 0x2B
#define MAX98091_HPCONTROL_MIXHP_LSEL_BASE 4
#define MAX98091_HPCONTROL_MIXHP_LSEL_WIDTH 1
#define MAX98091_HPCONTROL_MIXHP_RSEL_BASE 5
#define MAX98091_HPCONTROL_MIXHP_RSEL_WIDTH 1

#define MAX98091_LEFT_HP_VOLUME 0x2C
#define MAX98091_LEFT_HP_VOLUME_HPVOLL_BASE 0
#define MAX98091_LEFT_HP_VOLUME_HPVOLL_WIDTH 5
#define MAX98091_LEFT_HP_VOLUME_HPLM_BASE 7
#define MAX98091_LEFT_HP_VOLUME_HPLM_WIDTH 1

#define MAX98091_RIGHT_HP_VOLUME 0x2D
#define MAX98091_RIGHT_HP_VOLUME_HPVOLR_BASE 0
#define MAX98091_RIGHT_HP_VOLUME_HPVOLR_WIDTH 5
#define MAX98091_RIGHT_HP_VOLUME_HPRM_BASE 7
#define MAX98091_RIGHT_HP_VOLUME_HPRM_WIDTH 1

#define MAX98091_LEFT_SPK_MIXER 0x2E
#define MAX98091_LEFT_SPK_MIXER_LEFT_DAC_BASE 0
#define MAX98091_LEFT_SPK_MIXER_LEFT_DAC_WIDTH 1
#define MAX98091_LEFT_SPK_MIXER_RIGHT_DAC_BASE 1
#define MAX98091_LEFT_SPK_MIXER_RIGHT_DAC_WIDTH 1
#define MAX98091_LEFT_SPK_MIXER_LINE_A_BASE 2
#define MAX98091_LEFT_SPK_MIXER_LINE_A_WIDTH 1
#define MAX98091_LEFT_SPK_MIXER_LINE_B_BASE 3
#define MAX98091_LEFT_SPK_MIXER_LINE_B_WIDTH 1

#define MAX98091_RIGHT_SPK_MIXER 0x2F
#define MAX98091_RIGHT_SPK_MIXER_LEFT_DAC_BASE 0
#define MAX98091_RIGHT_SPK_MIXER_LEFT_DAC_WIDTH 1
#define MAX98091_RIGHT_SPK_MIXER_RIGHT_DAC_BASE 1
#define MAX98091_RIGHT_SPK_MIXER_RIGHT_DAC_WIDTH 1
#define MAX98091_RIGHT_SPK_MIXER_LINE_A_BASE 2
#define MAX98091_RIGHT_SPK_MIXER_LINE_A_WIDTH 1
#define MAX98091_RIGHT_SPK_MIXER_LINE_B_BASE 3
#define MAX98091_RIGHT_SPK_MIXER_LINE_B_WIDTH 1

#define MAX98091_LEFT_SPK_VOLUME 0x31
#define MAX98091_LEFT_SPK_VOLUME_SPVOLL_BASE 0
#define MAX98091_LEFT_SPK_VOLUME_SPVOLL_WIDTH 6
#define MAX98091_LEFT_SPK_VOLUME_SPLM_BASE 7
#define MAX98091_LEFT_SPK_VOLUME_SPLM_WIDTH 1

#define MAX98091_RIGHT_SPK_VOLUME 0x32
#define MAX98091_RIGHT_SPK_VOLUME_SPVOLR_BASE 0
#define MAX98091_RIGHT_SPK_VOLUME_SPVOLR_WIDTH 6
#define MAX98091_RIGHT_SPK_VOLUME_SPRM_BASE 7
#define MAX98091_RIGHT_SPK_VOLUME_SPRM_WIDTH 1

#define MAX98091_JACK_DETECT 0x3D

#define MAX98091_INPUT_ENABLE 0x3E

#define MAX98091_OUTPUT_ENABLE 0x3F

#define MAX98091_DSP_FILTER_ENABLE 0x41

#define MAX98091_BIAS_CONTROL 0x42

#define MAX98091_DAC_CONTROL 0x43

#define MAX98091_ADC_CONTROL 0x44

#define MAX98091_DEVICE_SHUTDOWN 0x45